#!/usr/bin/python
from __future__ import absolute_import, division, print_function
# Copyright 2019-2020 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fmgr_dvmdb_script_obj
short_description: Script table.
description:
    - This module is able to configure a FortiManager device by allowing the
      user to [ delete get set update add ] the following apis.
    - /dvmdb/adom/{adom}/script/{script}
    - /dvmdb/global/script/{script}
    - /dvmdb/script/{script}
    - Examples include all parameters and values need to be adjusted to data sources before usage.

version_added: "2.10"
author:
    - Frank Shen (@fshen01)
    - Link Zheng (@zhengl)
notes:
    - There are only three top-level parameters where 'method' is always required
      while other two 'params' and 'url_params' can be optional
    - Due to the complexity of fortimanager api schema, the validation is done
      out of Ansible native parameter validation procedure.
    - The syntax of OPTIONS doen not comply with the standard Ansible argument
      specification, but with the structure of fortimanager API schema, we need
      a trivial transformation when we are filling the ansible playbook
options:
    loose_validation:
        description:
          - Do parameter validation in a loose way
        type: bool
        required: false
    workspace_locking_adom:
        description:
          - the adom name to lock in case FortiManager running in workspace mode
          - it can be global or any other custom adom names
        required: false
        type: str
    workspace_locking_timeout:
        description:
          - the maximum time in seconds to wait for other user to release the workspace lock
        required: false
        type: int
        default: 300
    method:
        description:
          - The method in request
        required: true
        type: str
        choices:
          - delete
          - get
          - set
          - update
          - add
    params:
        description:
          - The parameters for each method
          - See full parameters list in https://ansible-galaxy-fortimanager-docs.readthedocs.io/en/latest
        type: list
        required: false
    url_params:
        description:
          - The parameters for each API request URL
          - Also see full URL parameters in https://ansible-galaxy-fortimanager-docs.readthedocs.io/en/latest
        required: false
        type: dict

'''

EXAMPLES = '''
 - hosts: fortimanager-inventory
   collections:
     - fortinet.fortimanager
   connection: httpapi
   vars:
      ansible_httpapi_use_ssl: True
      ansible_httpapi_validate_certs: False
      ansible_httpapi_port: 443
   tasks:

    - name: REQUESTING /DVMDB/SCRIPT/{SCRIPT}
      fmgr_dvmdb_script_obj:
         loose_validation: False
         workspace_locking_adom: <value in [global, custom adom]>
         workspace_locking_timeout: 300
         method: <value in [delete]>
         url_params:
            adom: <value in [none, global, custom dom]>
            script: <value of string>
         params:
            -
               data:
                 -
                     name: <value of string>
                     vdom: <value of string>

    - name: REQUESTING /DVMDB/SCRIPT/{SCRIPT}
      fmgr_dvmdb_script_obj:
         loose_validation: False
         workspace_locking_adom: <value in [global, custom adom]>
         workspace_locking_timeout: 300
         method: <value in [get]>
         url_params:
            adom: <value in [none, global, custom dom]>
            script: <value of string>
         params:
            -
               option: <value in [object member, chksum]>

    - name: REQUESTING /DVMDB/SCRIPT/{SCRIPT}
      fmgr_dvmdb_script_obj:
         loose_validation: False
         workspace_locking_adom: <value in [global, custom adom]>
         workspace_locking_timeout: 300
         method: <value in [set, update]>
         url_params:
            adom: <value in [none, global, custom dom]>
            script: <value of string>
         params:
            -
               data:
                  content: <value of string>
                  desc: <value of string>
                  filter_build: <value of integer>
                  filter_device: <value of integer>
                  filter_hostname: <value of string>
                  filter_ostype: <value in [unknown, fos]>
                  filter_osver: <value in [unknown, 4.00, 5.00]>
                  filter_platform: <value of string>
                  filter_serial: <value of string>
                  modification_time: <value of string>
                  name: <value of string>
                  script_schedule:
                    -
                        datetime: <value of string>
                        day_of_week: <value in [unknown, sun, mon, ...]>
                        device: <value of integer>
                        name: <value of string>
                        run_on_db: <value in [disable, enable]>
                        type: <value in [auto, onetime, daily, ...]>
                  target: <value in [device_database, remote_device, adom_database]>
                  type: <value in [cli, tcl, cligrp]>

    - name: REQUESTING /DVMDB/SCRIPT/{SCRIPT}
      fmgr_dvmdb_script_obj:
         loose_validation: False
         workspace_locking_adom: <value in [global, custom adom]>
         workspace_locking_timeout: 300
         method: <value in [set, update]>
         url_params:
            adom: <value in [none, global, custom dom]>
            script: <value of string>
         params:
            -
               data:
                 -
                     name: <value of string>
                     vdom: <value of string>

    - name: REQUESTING /DVMDB/SCRIPT/{SCRIPT}
      fmgr_dvmdb_script_obj:
         loose_validation: False
         workspace_locking_adom: <value in [global, custom adom]>
         workspace_locking_timeout: 300
         method: <value in [add]>
         url_params:
            adom: <value in [none, global, custom dom]>
            script: <value of string>
         params:
            -
               data:
                 -
                     name: <value of string>
                     vdom: <value of string>

'''

RETURN = '''
url:
    description: The full url requested
    returned: always
    type: str
    sample: /sys/login/user
status:
    description: The status of api request
    returned: always
    type: dict
data:
    description: The payload returned in the request
    type: dict
    returned: always

'''
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import Connection
from ansible_collections.fortinet.fortimanager.plugins.module_utils.common import FAIL_SOCKET_MSG
from ansible_collections.fortinet.fortimanager.plugins.module_utils.common import DEFAULT_RESULT_OBJ
from ansible_collections.fortinet.fortimanager.plugins.module_utils.common import FMGRCommon
from ansible_collections.fortinet.fortimanager.plugins.module_utils.common import FMGBaseException
from ansible_collections.fortinet.fortimanager.plugins.module_utils.fortimanager import FortiManagerHandler


def main():
    jrpc_urls = [
        '/dvmdb/adom/{adom}/script/{script}',
        '/dvmdb/global/script/{script}',
        '/dvmdb/script/{script}'
    ]

    url_schema = [
        {
            'name': 'adom',
            'type': 'string'
        },
        {
            'name': 'script',
            'type': 'string'
        }
    ]

    body_schema = {
        'schema_objects': {
            'object0': [
                {
                    'type': 'string',
                    'name': 'url',
                    'api_tag': 0
                },
                {
                    'name': 'data',
                    'type': 'dict',
                    'dict': {
                        'type': 'array',
                        'items': {
                            'name': {
                                'type': 'string'
                            },
                            'vdom': {
                                'type': 'string'
                            }
                        }
                    },
                    'api_tag': 1
                },
                {
                    'type': 'string',
                    'name': 'url',
                    'api_tag': 1
                }
            ],
            'object1': [
                {
                    'name': 'option',
                    'type': 'dict',
                    'dict': {
                        'type': 'string',
                        'enum': [
                            'object member',
                            'chksum'
                        ]
                    },
                    'api_tag': 0
                },
                {
                    'type': 'string',
                    'name': 'url',
                    'api_tag': 0
                }
            ],
            'object2': [
                {
                    'name': 'data',
                    'type': 'dict',
                    'dict': {
                        'content': {
                            'type': 'string'
                        },
                        'desc': {
                            'type': 'string'
                        },
                        'filter_build': {
                            'type': 'integer'
                        },
                        'filter_device': {
                            'type': 'integer'
                        },
                        'filter_hostname': {
                            'type': 'string'
                        },
                        'filter_ostype': {
                            'type': 'string',
                            'enum': [
                                'unknown',
                                'fos'
                            ]
                        },
                        'filter_osver': {
                            'type': 'string',
                            'enum': [
                                'unknown',
                                '4.00',
                                '5.00'
                            ]
                        },
                        'filter_platform': {
                            'type': 'string'
                        },
                        'filter_serial': {
                            'type': 'string'
                        },
                        'modification_time': {
                            'type': 'string'
                        },
                        'name': {
                            'type': 'string'
                        },
                        'script_schedule': {
                            'type': 'array',
                            'items': {
                                'datetime': {
                                    'type': 'string'
                                },
                                'day_of_week': {
                                    'type': 'string',
                                    'enum': [
                                        'unknown',
                                        'sun',
                                        'mon',
                                        'tue',
                                        'wed',
                                        'thu',
                                        'fri',
                                        'sat'
                                    ]
                                },
                                'device': {
                                    'type': 'integer'
                                },
                                'name': {
                                    'type': 'string'
                                },
                                'run_on_db': {
                                    'type': 'string',
                                    'enum': [
                                        'disable',
                                        'enable'
                                    ]
                                },
                                'type': {
                                    'type': 'string',
                                    'enum': [
                                        'auto',
                                        'onetime',
                                        'daily',
                                        'weekly',
                                        'monthly'
                                    ]
                                }
                            }
                        },
                        'target': {
                            'type': 'string',
                            'enum': [
                                'device_database',
                                'remote_device',
                                'adom_database'
                            ]
                        },
                        'type': {
                            'type': 'string',
                            'enum': [
                                'cli',
                                'tcl',
                                'cligrp'
                            ]
                        }
                    },
                    'api_tag': 0
                },
                {
                    'type': 'string',
                    'name': 'url',
                    'api_tag': 0
                },
                {
                    'name': 'data',
                    'type': 'dict',
                    'dict': {
                        'type': 'array',
                        'items': {
                            'name': {
                                'type': 'string'
                            },
                            'vdom': {
                                'type': 'string'
                            }
                        }
                    },
                    'api_tag': 1
                },
                {
                    'type': 'string',
                    'name': 'url',
                    'api_tag': 1
                }
            ],
            'object3': [
                {
                    'name': 'data',
                    'type': 'dict',
                    'dict': {
                        'type': 'array',
                        'items': {
                            'name': {
                                'type': 'string'
                            },
                            'vdom': {
                                'type': 'string'
                            }
                        }
                    },
                    'api_tag': 0
                },
                {
                    'type': 'string',
                    'name': 'url',
                    'api_tag': 0
                }
            ]
        },
        'method_mapping': {
            'delete': 'object0',
            'get': 'object1',
            'set': 'object2',
            'update': 'object2',
            'add': 'object3'
        }
    }

    module_arg_spec = {
        'loose_validation': {
            'type': 'bool',
            'required': False,
            'default': False
        },
        'workspace_locking_adom': {
            'type': 'str',
            'required': False
        },
        'workspace_locking_timeout': {
            'type': 'int',
            'required': False,
            'default': 300
        },
        'params': {
            'type': 'list',
            'required': False
        },
        'method': {
            'type': 'str',
            'required': True,
            'choices': [
                'delete',
                'get',
                'set',
                'update',
                'add'
            ]
        },
        'url_params': {
            'type': 'dict',
            'required': False
        }
    }
    module = AnsibleModule(argument_spec=module_arg_spec,
                           supports_check_mode=False)
    method = module.params['method']
    loose_validation = module.params['loose_validation']

    fmgr = None
    payload = None
    response = DEFAULT_RESULT_OBJ

    if module._socket_path:
        connection = Connection(module._socket_path)
        tools = FMGRCommon()
        if loose_validation is False:
            tools.validate_module_params(module, body_schema)
        tools.validate_module_url_params(module, jrpc_urls, url_schema)
        full_url = tools.get_full_url_path(module, jrpc_urls)
        payload = tools.get_full_payload(module, full_url)
        fmgr = FortiManagerHandler(connection, module)
        fmgr.tools = tools
    else:
        module.fail_json(**FAIL_SOCKET_MSG)

    try:
        response = fmgr._conn.send_request(method, payload)
        fmgr.govern_response(module=module, results=response,
                             msg='Operation Finished',
                             ansible_facts=fmgr.construct_ansible_facts(response, module.params, module.params))
    except Exception as e:
        raise FMGBaseException(e)

    module.exit_json(meta=response[1])


if __name__ == '__main__':
    main()
